package com.adzel.velocitybroadcast.util;

import java.nio.file.Path;
import java.sql.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Logger;

import com.adzel.velocitybroadcast.ConfigHandler;

public class DatabaseManager {
    private final Logger logger;
    private final ConfigHandler config;
    private final Path dataDirectory;
    private Connection connection;
    private final ExecutorService executor = Executors.newSingleThreadExecutor();

    public DatabaseManager(ConfigHandler config, Logger logger, Path dataDirectory) {
        this.config = config;
        this.logger = logger;
        this.dataDirectory = dataDirectory;
    }

    public void initialize() {
        try {
            if (config.getDbType().equalsIgnoreCase("mysql")) {
                Class.forName("com.mysql.cj.jdbc.Driver");
                connection = DriverManager.getConnection(
                    "jdbc:mysql://" + config.getDbHost() + ":" + config.getDbPort() + "/" + config.getDbName() + "?useSSL=false&autoReconnect=true",
                    config.getDbUser(),
                    config.getDbPassword()
                );
            } else {
                Class.forName("org.sqlite.JDBC");
                Path dbFile = dataDirectory.resolve(config.getDbName() + ".db");
                connection = DriverManager.getConnection("jdbc:sqlite:" + dbFile.toAbsolutePath());
            }

            logger.info("[VelocityBroadcast] Connected to " + config.getDbType().toUpperCase() + " database.");
            initializeTables();

        } catch (Exception e) {
            logger.warning("[VelocityBroadcast] Failed to connect to database: " + e.getMessage());
        }
    }

    private void initializeTables() {
        boolean isMySQL = config.getDbType().equalsIgnoreCase("mysql");
        String idSyntax = isMySQL ? "INT AUTO_INCREMENT PRIMARY KEY" : "INTEGER PRIMARY KEY AUTOINCREMENT";

        String broadcastTable = "CREATE TABLE IF NOT EXISTS vb_broadcast_logs (" +
                "id " + idSyntax + "," +
                "timestamp DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP," +
                "sender VARCHAR(64) NOT NULL," +
                "message TEXT NOT NULL," +
                "source_server VARCHAR(64)" +
                ");";

        String commandTable = "CREATE TABLE IF NOT EXISTS vb_command_logs (" +
                "id " + idSyntax + "," +
                "timestamp DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP," +
                "user VARCHAR(64) NOT NULL," +
                "command TEXT NOT NULL," +
                "success BOOLEAN" +
                ");";

        String joinTable = "CREATE TABLE IF NOT EXISTS vb_admin_joins (" +
                "id " + idSyntax + "," +
                "timestamp DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP," +
                "username VARCHAR(64) NOT NULL," +
                "server VARCHAR(64)" +
                ");";

        executeUpdate(broadcastTable);
        executeUpdate(commandTable);
        executeUpdate(joinTable);
    }

    public void logBroadcast(String sender, String message, String sourceServer) {
        String sql = "INSERT INTO vb_broadcast_logs (sender, message, source_server) VALUES (?, ?, ?)";
        executeAsync(sql, sender, message, sourceServer);
    }

    public void logCommand(String user, String command, boolean success) {
        String sql = "INSERT INTO vb_command_logs (user, command, success) VALUES (?, ?, ?)";
        executeAsync(sql, user, command, success);
    }

    public void logAdminJoin(String username, String serverName) {
        String sql = "INSERT INTO vb_admin_joins (username, server) VALUES (?, ?)";
        executeAsync(sql, username, serverName);
    }

    private void executeAsync(String sql, Object... params) {
        executor.submit(() -> {
            try (PreparedStatement stmt = connection.prepareStatement(sql)) {
                for (int i = 0; i < params.length; i++) {
                    stmt.setObject(i + 1, params[i]);
                }
                stmt.executeUpdate();
            } catch (SQLException e) {
                logger.warning("[VelocityBroadcast] Database query failed: " + e.getMessage());
            }
        });
    }

    private void executeUpdate(String sql) {
        executor.submit(() -> {
            try (Statement stmt = connection.createStatement()) {
                stmt.executeUpdate(sql);
            } catch (SQLException e) {
                logger.warning("[VelocityBroadcast] Failed to initialize table: " + e.getMessage());
            }
        });
    }

    public void close() {
        try {
            if (connection != null && !connection.isClosed()) {
                connection.close();
                executor.shutdown();
            }
        } catch (SQLException e) {
            logger.warning("[VelocityBroadcast] Failed to close database: " + e.getMessage());
        }
    }
}
